#!/usr/bin/env python

# manifestops - Provides functions for performing different operations on a
# .manifest file
#
# Author: Danilo Segan <danilo@canonical.com>, with modifications by David
# Planella <david.planella@ubuntu.com> and Kyle Nitzsche 
# <kyle.nitzsche@canonical.com>
#
# Copyright (c) 2009 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import codecs
import os
import urllib2
from optparse import OptionParser
import datetime
import tempfile
import atexit
import shutil
import gzip
sys.path.append(os.path.join(os.getcwd(), 'lib/python2.6/site-packages'))
from gviz_api import gviz_api
from html_report_template import page_template
import utils


# To hold all statistics.
stats = {}

def addToLanguage(language, translated, total):
    if language in stats:
        stats[language]['translated'] += translated
        stats[language]['total'] += total
    else:
        stats[language] = {
            'translated': translated,
            'total': total,
            }

def do_lp_login():

    from launchpadlib.launchpad import Launchpad

    cachedir = tempfile.mkdtemp()
    atexit.register(shutil.rmtree, cachedir)

    print >> sys.stderr, "Logging in to Launchpad... "

    # TODO: error checking, especially if we're working offline
    launchpad = Launchpad.login_anonymously('Ubuntu Translation stats',
            'production', cachedir)

    return launchpad

def get_lang_names():

    max_tries = 2
    launchpad = do_lp_login()

    lang_names = {}

    print >>sys.stderr, 'Getting language list from Launchpad...'

    try:
        langs = launchpad.languages
    except urllib2.URLError, e:
        tries += 1
        if tries >= max_tries or (e.code != 502 and e.code != 504):
            print >>sys.stderr, "Failed (%d): %s" % (e.code, url)
            raise

        print >>sys.stderr, "Retrying (%d): %s" % (e.code, url)
        time.sleep(3)

    for lang in langs:
        if lang.visible:
            lang_names[lang.code] = lang.english_name

    print >>sys.stderr, '{0} languages obtained'.format(str(len(langs)))

    return lang_names


def print_report(distro_codename):
    basename = '{0}-translation-stats.html'.format(distro_codename)
    report_filename = os.path.join('reports', basename)

    f = codecs.open(report_filename, 'w', 'utf-8')
    f.write(page_template % vars())
    f.close()


def calculate_stats(options, settings, pot_set):
    #if sys.stdout.encoding is None:
    #    sys.stdout = codecs.open("/dev/stdout", "w", 'utf-8')

    # set up some vars
    pkg_stats = options.pkg_stats

    TOTAL = 0
    working_templates = set()

    for line in pot_set:
        (pkg, domain, pot, total, enabled, langpack, priority,
            date) = line.split('|')
        pkg = pkg.strip()
        pot = pot.strip()
        domain = domain.strip()
        total = int(total.strip())
        working_templates.add((pkg, domain))
        TOTAL += total
    #print >> sys.stderr, "SRC PKGs: " + str(len(open(working_pot_stats).readlines()))
    print >> sys.stderr, "SRC PKGs AFTER BLACKLIST: " + str(len(working_templates))
    print >> sys.stderr, "TOTAL MESSAGES: " + str(TOTAL)

    lang_names = get_lang_names()

    # Skip 2 lines from the package stats file
    started = 2

    # Read all stats from the package stats file
    for line in gzip.open(pkg_stats, 'rb').readlines():
        if started > 0:
            started -= 1
        elif '|' in line:
            (pkg, domain, pot, date_pot, date_translation, lang, translated,
                unreviewed, changed, total) = line.split('|')
            pkg = pkg.strip()
            pot = pot.strip()
            domain = domain.strip()
            lang = lang.strip()
            translated = int(translated.strip())
            total = int(total.strip())

            if (pkg, domain) in working_templates and lang in lang_names:
                addToLanguage(lang, translated, total)
            #print >> sys.stderr, "%(lang)s, %(trans)d, %(total)d " \
            #    % {'lang': lang, 'trans': translated, 'total':total}

    # create langs and sort from most translated to least 
    langs = stats.keys()
    #print >> sys.stderr, "LANGS: " + str (langs)
    langs.sort(cmp=lambda a, b: cmp(stats[b]['translated'], stats[a]['translated']))
    #print >> sys.stderr, "LANGS, sorted: " + str(langs)

    translated_langs = []
    langs_list = []
    total_langs = 0

    for lang in langs:
        if lang in lang_names:
            if stats[lang]['translated'] == 0:
                percent = 0
            else:
                percent = int(round(100.00 * stats[lang]['translated'] / TOTAL))
            langs_list.append({
                "language": lang_names[lang],
                "translated": int(percent)
                })
        if percent > 0:
            translated_langs.append(lang_names[lang])

    translated_langs.sort()

    # Log the date of creation for the report
    gen_date = str(datetime.datetime.utcnow())
    total_translated_langs = str(len(translated_langs))
    #translated_langs_str = "(" + total_translated_langs + ") " + ", ".join(translated_langs)
    translated_langs_str = "({0}) ".format(total_translated_langs) + ", ".join(translated_langs)
    total_langs = str(len(langs_list))

    keys = ("language", "translated")
    description = {"language": ("string", "Language"),
        "translated": ("number", "Translated")}

    # Load the data it into gviz_api.DataTable
    data_table = gviz_api.DataTable(description)
    data_table.LoadData(langs_list)

    # FIXME: If we do rounding of the percentages before here, languages with
    # the same percentage after rounding will be simply ordered alphabetically.
    # When losing the figures after the decimal point, we lose the order of
    # languages in a given percentage range
    languages_json = data_table.ToJSon(columns_order=("language", "translated"),
                                        order_by=[("translated", "desc"), ("language", "asc")])

    # Put the JSon string into the template
    distro_release = settings['distro_release']
    if not options.custom:
        distro_codename = settings['distro_codename']
    else:
        distro_codename = "Custom"

    ## TODO: pass all the required variables and do the reporting in a function
    #print_report(distro_codename)

    basename = '{0}-translation-stats.html'.format(distro_codename)
    report_filename = os.path.join('reports', basename)

    f = codecs.open(report_filename, 'w', 'utf-8')
    f.write(page_template % vars())
    f.close()
